/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.manager.requests;


import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.manager.ManagerErrorMessage;
import de.bsvrz.pua.prot.manager.datamanager.DataManagerInterface;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;

import java.io.File;

/**
 * Auftrag "Skript löschen". Siehe Feinentwurf Kapitel 2.1.2.2.1.2 Feinentwurf ändern: Skript wird vom client aus objektmenge entfernt. -&gt; löschversuch der
 * daten. wenn die Daten gelöscht werden konnten, wird das skript von pua mit .invalidate gelöscht
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class DeleteScriptRequest extends ScriptRequest {

	/** Objekte, das entfernt wird. */
	private SystemObject _remove;

	/**
	 * @param dav         Verbindung zum Datenverteiler.
	 * @param dataManager Daten Manager Submodul.
	 * @param remove      Objekte, das entfernt wird.
	 */
	public DeleteScriptRequest(ClientDavInterface dav, DataManagerInterface dataManager, SystemObject remove) {
		super(dav, dataManager, new ResultData[]{}, false);
		this._remove = remove;
	}

	/**
	 * Löscht die Metainformationen und den Skriptquelltext.
	 *
	 * @see de.bsvrz.pua.prot.manager.requests.DataManagerRequest#process()
	 */
	@Override
    public ProtocolRequestResult process() {
		boolean deleted = true;

		deleted &= delete(new File(dataManager.getAbsolutFileNameMeta(_remove)));
		deleted &= delete(new File(dataManager.getAbsolutFileNameSource(_remove)));
		try {
			_remove.invalidate();
		}
		catch(ConfigurationChangeException e) {
			debug.error("Objekt konnte nicht ungültig gemacht werden: " + script);
		}

		if(deleted) {
			return new ProtocolRequestResult();
		}
		else {
			return new ProtocolRequestResult(ManagerErrorMessage.CAN_NOT_DELETE_FILE);
		}
	}

	/**
	 * Löscht eine Datei. Fängt alle Exceptions. Schlägt das Löschen fehl, so wird gewartet und es noch zwei weitere Male versucht.
	 *
	 * @param file Datei die gelöscht wird.
	 *
	 * @return Ergebnis des Löschvorgangs. Wie bei {@link File#delete()}.
	 */
	private boolean delete(File file) {
		boolean result = false;
		try {
			for(int i = 0; i < 3 && !result; i++) {
				result = file.delete();
				if(!result) Thread.sleep((i + 1) * 10);
			}
			if(result) {
				debug.info("Datei gelöscht: " + file.getAbsolutePath() + File.separator + file.getName());
			}
			else {
				debug.error(ManagerErrorMessage.CAN_NOT_DELETE_FILE + file.getAbsolutePath() + File.separator + file.getName());
			}
		}
		catch(Exception e) {
			debug.error(ManagerErrorMessage.CAN_NOT_DELETE_FILE + file.getAbsolutePath() + File.separator + file.getName(), e);
		}
		return result;
	}
}
