/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.manager.status;

import de.bsvrz.pua.prot.manager.taskmanager.ThreadElement;

import java.util.TimerTask;

/**
 * Abstrakte Basisklasse für Module, die Anwender über interne Vorgänge in PuA informieren.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 12804 $
 */
public abstract class StatePublisherTask extends TimerTask {

	private StatePublisher _statePublisher;

	public abstract int getInterval();

	public final void setDataSource(final StatePublisher statePublisher) {
		_statePublisher = statePublisher;
	}

	@Override
	public void run() {
		update(_statePublisher.getStatus());
	}

	/**
	 * Wird periodisch (siehe {@link #getInterval}) aufgerufen, um über den aktuellen Status zu informieren
	 *
	 * @param status aktueller Status
	 */
	protected abstract void update(final StatePublisher.Status status);

	/**
	 * Wird aufgerufen, wenn sich ein Auftrag beendet hat
	 * @param threadElement Auftrag
	 */
	public abstract void notifyThreadFinished(final ThreadElement threadElement);

	/**
	 * Wird aufgerufen, nachdem ein Protokoll gelöscht wird
	 * @param protocolId Protokoll-Id
	 */
	public abstract void notifyProtocolRemoved(final long protocolId);

	/**
	 * Wird aufgerufen, wenn ein Protokoll den Status von gelesen nach ungelesen ändert
	 * @param protocolId Protokoll-Id
	 */
	public abstract void notifyProtocolRead(final long protocolId);

	/**
	 * Wird aufgerufen, wenn sich der Zustand des Archivsystems ändert
	 * @param archiveAvailable Ob das Archivsystem verfügbar ist, oder nicht
	 */
	public abstract void notifyArchiveAvailable(boolean archiveAvailable);
}
