/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.util.attributes;

import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.pua.prot.aggregations.Aggregation;
import de.bsvrz.pua.prot.util.Qualifier;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

import java.util.*;

/**
 * Repräsentiert eine Attributgruppe
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class AttributeGroupDescription extends Elements implements RealElement {

	/**
	 * <code>true</code> falls Daten durch die Datenaufbereitung aufgefüllt werden sollen. Wird in {@link #equals(Object)} und {@link #hashCode()} nicht
	 * berücksichtigt.
	 */
	private boolean _padData = true;

	/** Qualifizierung der Attributgruppe */
	public Qualifier qualifier;

	private int _index = -1;

	/**
	 * @param simVar  Simulationsvariante.
	 * @param padData <code>true</code> falls Daten durch die Datenaufbereitung aufgefüllt werden sollen.
	 */
	public AttributeGroupDescription(short simVar, boolean padData) {
		this.qualifier = new Qualifier(simVar);
		this._padData = padData;
	}

	/**
	 * Test-Konstruktor. Daten werden aufgefüllt.
	 *
	 * @param model       Konfiguration
	 * @param so          Systemobjekt
	 * @param atg         Attributgruppe
	 * @param asp         Aspekt
	 * @param simVar      Simulationsvariante
	 * @param columnName  Spaltenname
	 * @param aggregations Aggregation
	 *
	 * @throws FailureException Ungültige Qualifizierung.
	 */
	public AttributeGroupDescription(DataModel model, String so, String atg, String asp, short simVar, String columnName, List<Class<? extends Aggregation>> aggregations)
			throws FailureException {
		this.qualifier = new Qualifier(model, so, atg, asp, simVar);
		this.setColumnName(columnName);
		this.setAggregationClasses(aggregations);
	}

	/**
	 * Gibt den verwendeten Datenindex zurück
	 *
	 * @return Datenindex der Attributgruppe
	 */
	@Override
    public Collection<Qualifier> getQualifiers() {
		HashSet<Qualifier> h = new HashSet<Qualifier>();
		h.add(qualifier);

		return h;
	}

	/** @see java.lang.Object#toString() */
	@Override
    public String toString() {
		return "Attributgruppe: " + qualifier.toString();
	}

	/** @see java.lang.Object#equals(java.lang.Object) */
	@Override
    public boolean equals(Object obj) {
		boolean result = false;
		if(obj != null && obj instanceof AttributeGroupDescription) {
			AttributeGroupDescription atg = (AttributeGroupDescription)obj;
			result = Objects.equals(this.qualifier, atg.qualifier) && super.equals(atg) && this._padData == atg._padData;
		}
		return result;
	}

	/**
	 * Gibt eine Referenz auf dieses Objekt zurück.
	 *
	 * @see de.bsvrz.pua.prot.util.attributes.Elements#getRealElements()
	 */
	@Override
	public HashSet<AttributeGroupDescription> getRealElements() {
		HashSet<AttributeGroupDescription> h = new HashSet<AttributeGroupDescription>(1);
		h.add(this);
		return h;
	}

	/**
	 * Klont die Attributgruppenbeschreibung. Dabei wird eine neue Aggregation angelegt, d.h. der Zustand der Aggregation geht verloren.
	 *
	 * @return Kopie der Attributgruppenbeschreibung
	 *
	 * @see java.lang.Object#clone()
	 */
	@Override
    public AttributeGroupDescription clone() {
		AttributeGroupDescription clone = new AttributeGroupDescription(this.qualifier.simVar, this._padData);

		if(qualifier != null) clone.qualifier = qualifier.clone();
		if(getColumnName() != null) clone.setColumnName(getColumnName());

		clone.setAggregationClasses(getAggregationClasses());
		return clone;
	}


	/** @see de.bsvrz.pua.prot.util.attributes.RealElement#padData() */
	public boolean padData() {
		return _padData;
	}

	public void setIndex(final int index) {
		_index = index;
	}

	public int getIndex() {
		return _index;
	}
}
